@@ -5,6 +5,7 @@ from rest_framework.response import Response
 from rest_framework import generics
 from rest_framework.permissions import IsAuthenticated, IsAdminUser, AllowAny
 
+from utils.custom_tool import request_params_wrapper
 from ..services.finance_transacts_service import FinanceTransactsService
 from xj_user.services.user_service import UserService
 from ..utils.model_handle import parse_data, util_response
@@ -99,9 +100,21 @@ class FinanceTransact(generics.UpdateAPIView):  # 或继承(APIView)
 
     def finance_flow_writing(self):
         params = parse_data(self)
-        # print(params)
-        data, err_txt = FinanceTransactService.finance_flow_writing(params=params,
-                                                                    finance_type='PAY')
+        # data, err_txt = FinanceTransactService.finance_flow_writing(params=params,
+        #                                                             finance_type='PAY')
+        data, err_txt = FinanceTransactsService.finance_create_or_write_off(params)
+        if err_txt is None:
+            return util_response(data=data)
+        return util_response(err=47767, msg=err_txt)
+
+    @request_params_wrapper
+    def finance_standing_book(self, *args, user_info=None, request_params=None, **kwargs):
+        # params = parse_data(self)
+        if request_params is None:
+            request_params = {}
+        # data, err_txt = FinanceTransactService.finance_flow_writing(params=params,
+        #                                                             finance_type='PAY')
+        data, err_txt = FinanceTransactsService.finance_standing_book(request_params)
         if err_txt is None:
             return util_response(data=data)
         return util_response(err=47767, msg=err_txt)
