@@ -1,25 +1,27 @@
 import json
+import re
 import sys
 from decimal import Decimal
 import random
+from tkinter import E
 
 import pytz
 from django.core.paginator import Paginator
-from django.db.models import Q
+from django.db.models import Q, Sum, F
 from django.forms import model_to_dict
 from rest_framework import serializers
 from pathlib import Path
 from main.settings import BASE_DIR
 
 from xj_enroll.utils.custom_tool import format_params_handle
-from xj_finance.services.finance_transact_service import FinanceTransactService
+from xj_finance.services.finance_transact_service import FinanceTransactService, DateEncoder
 from xj_finance.utils.custom_tool import filter_result_field
 from xj_thread.utils.join_list import JoinList
 from xj_user.models import BaseInfo
 from xj_user.services.user_platform_service import UserPlatformService
 from xj_thread.services.thread_list_service import ThreadListService
 from .finance_service import FinanceService
-from ..models import Transact
+from ..models import Transact, PayMode
 from ..utils.j_config import JConfig
 from ..utils.j_dict import JDict
 from logging import getLogger
@@ -83,6 +85,7 @@ class FinanceTransactsSerializer(serializers.ModelSerializer):
             # 'transact_no',
             'transact_no',
             "thread_id",
+            "enroll_id",
             'transact_time',
             # 'transact_timestamp',
             'platform_id',
@@ -158,6 +161,11 @@ class FinanceTransactsService:
         else:
             transacts = Transact.objects.filter(account_id=user_id).filter(**valid['query_dict'])
 
+        if params.get("is_enroll", None):
+            transacts = Transact.objects.filter(enroll_id__isnull=False).filter(**valid['query_dict'])
+        if params.get("is_thread", None):
+            transacts = Transact.objects.filter(thread_id__isnull=False).filter(**valid['query_dict'])
+
         transacts = transacts.order_by('-transact_time')
         # print(params['transact_time_start'])
         if params.get("transact_time_start", None) and params.get("transact_time_end", None):
@@ -186,6 +194,8 @@ class FinanceTransactsService:
         size = int(params['size']) if 'size' in params else 10
 
         total = transacts.count()
+        income = transacts.aggregate(income=Sum("income"))
+        outgo = transacts.aggregate(outgo=Sum("outgo"))
 
         current_page_set = transacts[page * size: page * size + size] if page >= 0 and size > 0 else transacts
 
@@ -198,8 +208,23 @@ class FinanceTransactsService:
             it['platform_name'] = params.get('platform', '')
             res_list.append(it)
         # return {'total': total, 'list': res_list, 'statistics': statistic_list}, None
+        data = res_list
+        thread_id_list = [item.get("thread_id", None) for item in res_list]
+        thread_list, err = ThreadListService.search(thread_id_list)
+        if thread_list:
+            data = JoinList(res_list, thread_list, "thread_id", "id").join()
 
-        return {'size': int(size), 'page': int(page + 1), 'total': total, 'list': res_list}, None
+        statistics = {
+            "income": income.get("income", 0.0) if income.get("income", 0.0) else 0.0,
+            "outgo": outgo.get("outgo", 0.0) if outgo.get("outgo", 0.0) else 0.0,
+        }
+        # enroll_id_list = [item.get("enroll_id", None) for item in res_list]
+        # thread_list, err = ThreadListService.search(thread_id_list)
+        # if thread_list:
+        #     data = JoinList(res_list, thread_list, "thread_id", "id").join()
+        # print(enroll_id_list)
+
+        return {'size': int(size), 'page': int(page + 1), 'total': total, 'list': data, "statistics": statistics}, None
 
         # return {'total': total, 'list': res_list}, None
         # 翻译
@@ -245,7 +270,7 @@ class FinanceTransactsService:
     @staticmethod
     def detail_all(order_no=None):
         """
-        查询订单性情
+        查询订单详情
         """
         if not order_no:
             return None, None
@@ -399,7 +424,7 @@ class FinanceTransactsService:
                 # 生成真实记录成功后 原沙盒记录改为核销
                 Transact.objects.filter(transact_no=transact_no).update(
                     finance_status_code=finance_status_code,
-                    sand_box_status_code="transfered",
+                    sand_box_status_code="TRANSFERED",
                     is_write_off=1,
                     is_reverse=None)  # 沙盒核销
             elif type == "refuse":
@@ -411,6 +436,15 @@ class FinanceTransactsService:
                     finance_status_code=615,
                     is_reverse=None)  # 沙盒核销
 
+        return None, None
+
+    @staticmethod
+    def finance_create_or_write_off(data):
+        data['action'] = "收入"
+        finance_order, err_txt = FinanceTransactService.finance_flow_writing(params=data, finance_type='PAY')
+        if finance_order:
+            params = {"order_no": finance_order, "type": "write_off"}
+            FinanceTransactsService.write_off(params)
 
         return None, None
 
@@ -476,7 +510,7 @@ class FinanceTransactsService:
             return None, err
         transact_list['list'] = filter_result_field(
             result_list=transact_list['list'],
-            alias_dict={"snapshot": "finance_snapshot"}
+            alias_dict={"snapshot": "snapshot"}
         )
         income = 0
         outgo = 0
@@ -506,44 +540,65 @@ class FinanceTransactsService:
     @staticmethod
     def invoicing_approval(params):
         finance_id = params.get("finance_id", None)
-        transact_no = params.get("transact_no", None)
+        goods_info = params.get("goods_info", None)
         if not finance_id:
             return None, "id不能为空"
         finance = Transact.objects.filter(id=finance_id)
         finance_data = finance.first()
+        if not finance_data:
+            return None, "数据不存在"
         finance_data = model_to_dict(finance_data)
-        goods_info = finance_data['goods_info']
-        enroll_list = []
-        jsDumps = json.dumps(goods_info['goods_info'])
+        finance_goods_info = finance_data['goods_info']
+        jsDumps = json.dumps(finance_goods_info)
         jsLoads = json.loads(jsDumps)
+        for i in goods_info:
+            before_key = i[0:i.rfind('__')]  # 截取指定字符前的字符串
+            behind_key = i.split('__')[-1]  # 截取指定字符后的字符串
+            if before_key in jsLoads:
+                object = jsLoads[before_key]
+                object[behind_key] = goods_info[i]
+                jsLoads[before_key] = object
+        # print(json.dumps(jsLoads))
+        finance.update(sand_box_status_code="INVOICED", goods_info=jsLoads)
+        enroll_list = []
+        # if 'enroll' in jsLoads:
+        #     for i in finance_goods_info['enroll']:
+        #         enroll_list.append(i['id'])
+        #     # enroll_list = FinanceTransactsService.get_json_primary_key(snapshot)
+        #     if not sys.modules.get("xj_enroll.service.enroll_services.EnrollServices"):
+        #         from xj_enroll.service.enroll_services import EnrollServices
+        #     EnrollServices.enroll_edit(params={"finance_invoicing_code": "INVOICED"},
+        #                                search_param={"enroll_id_list": enroll_list})
         if 'enroll' in jsLoads:
-            for i in goods_info['goods_info']['enroll']:
-                enroll_list.append(i['id'])
-            # enroll_list = FinanceTransactsService.get_json_primary_key(snapshot)
             if not sys.modules.get("xj_enroll.service.enroll_services.EnrollServices"):
                 from xj_enroll.service.enroll_services import EnrollServices
-            EnrollServices.enroll_edit(params={"finance_invoicing_code": "Invoiced"},
-                                       search_param={"enroll_id_list": enroll_list})
-
-            finance.update(sand_box_status_code="Invoiced", transact_no=transact_no)  # 沙盒核销
-
+            if isinstance(finance_goods_info['enroll'], dict):
+                EnrollServices.enroll_edit(params={"finance_invoicing_code": "INVOICING"},
+                                           search_param={"enroll_id": finance_goods_info['enroll']['id']})
+            else:
+                for i in finance_goods_info['enroll']:
+                    enroll_list.append(int(i['id']))
+                EnrollServices.enroll_edit(params={"finance_invoicing_code": "INVOICING"},
+                                           search_param={"enroll_id_list": enroll_list})
         return None, None
 
     @staticmethod
     def large_transfer(params):
         # print(params)
-        finance, err_txt = FinanceTransactService.finance_flow_writing(params=params, finance_type='PAY')
+        finance, err_txt = FinanceTransactService.finance_flow_writing(params=params, finance_type='TRANSFER')
         if err_txt:
             return None, err_txt
         return None, None
 
     @staticmethod
-    def large_transfer(params):
-        # print(params)
-        finance, err_txt = FinanceTransactService.finance_flow_writing(params=params, finance_type='PAY')
-        if err_txt:
-            return None, err_txt
-        return None, None
+    def large_amount_audit(params):
+        enroll_id = params.get("enroll_id", "")
+
+        transact_set = Transact.objects.filter(enroll_id=enroll_id, sand_box_status_code="TRANSFERING").first()
+        if transact_set:
+            return {"status": "1"}, None
+        else:
+            return {"status": "0"}, None
 
     # @staticmethod
     # def get_json_primary_key(json):
@@ -551,3 +606,60 @@ class FinanceTransactsService:
     #     for item in json:
     #         enroll_list.append(item['id'])
     #     return enroll_list
+
+    @staticmethod
+    def finance_standing_book(params):
+        enroll_id_list = params.get("enroll_id_list", None)
+        transact_obj = Transact.objects
+        list = []
+        for i in enroll_id_list:
+            transact_set = transact_obj.filter(enroll_id=i, sand_box__isnull=True, ).order_by("-id").first()
+            if not transact_set:
+                continue
+            finance_data = transact_set.to_dict()
+            pay_mode = PayMode.objects.filter(id=finance_data['pay_mode']).first()
+            pay_mode_data = model_to_dict(pay_mode)
+            standing_book = {}
+            standing_book['enroll_id'] = i  # 报名ID
+            standing_book['billing_time'] = None  # 开票时间
+            standing_book['charge_time'] = finance_data['transact_time']  # 收款时间时间
+            standing_book['charge_mode'] = pay_mode_data['pay_mode']  # 收款方式
+            standing_book['payment_time'] = None  # 付款时间 （暂无）
+            standing_book['payment_delay'] = None  # 付款方式（暂无）
+            standing_book['billing_time'] = None  # 开票时间
+
+            invoice_set = transact_obj.filter(
+                sand_box__sand_box_name__in=["BID_SPECIAL_INVOICE", "BID_PLAIN_INVOICE"], ).order_by("-id").values(
+                "goods_info")
+            if not invoice_set:
+                continue
+            for item in (invoice_set):
+                if item['goods_info']:
+                    # jsDumps = json.dumps(goods_info, cls=DateEncoder)
+                    # jsLoads = json.loads(jsDumps)
+                    # print(jsLoads)
+                    if "enroll" in item['goods_info']:
+                        enroll = item['goods_info']['enroll']
+                        if isinstance(enroll, dict):
+                            if enroll["id"] == i:
+                                if 'invoice' in item['goods_info']:
+                                    invoice = item['goods_info']['invoice']
+                                    billing_time = invoice.get("billing_time", None)
+                                    standing_book['billing_time'] = billing_time
+                        else:
+                            for enroll_item in enroll:
+                                if enroll_item["id"] == i:
+                                    if 'invoice' in item['goods_info']:
+                                        invoice = item['goods_info']['invoice']
+                                        billing_time = invoice.get("billing_time", None)
+                                        standing_book['billing_time'] = billing_time
+
+                                        # print(standing_book)
+            invoiced_amount = float(finance_data['income']) + float(finance_data['outgo'])
+            standing_book['invoiced_amount'] = abs(invoiced_amount)  # 发票金额
+            list.append(standing_book)
+
+        if list:
+            return list, None
+        else:
+            return {"status": "0"}, None
